(function ($) {
    'use strict';
 
    var SliderCaptcha = function (element, options) {
        this.$element = $(element);
        this.options = $.extend({}, SliderCaptcha.DEFAULTS, options);
        this.$element.css({ 'position': 'relative', 'width': this.options.width + 'px', 'margin': '0 auto' });
        console.log('options', options);
        console.log('optionsNew', this.options);
        this.init();
    };
 
    SliderCaptcha.VERSION = '1.0';
    SliderCaptcha.Author = 'argo@163.com';
    SliderCaptcha.DEFAULTS = {
        width: 280,     // canvas宽度
        height: 155,    // canvas高度
        PI: Math.PI,
        sliderL: 42,    // 滑块边长
        sliderR: 9,     // 滑块半径
        offset: 5,      // 容错偏差
        loadingText: '正在加载中...',
        failedText: '再试一次',
        barText: '向右滑动填充拼图',
        onFail: function () {
            
        },
        maxLoadCount: 3,
        localImages: function () {
            return 'images/Pic' + Math.round(Math.random() * 4) + '.jpg';
        }
    };
    function Plugin(option) {
        return this.each(function () {
            var $this = $(this);
            var data = $this.data('lgb.SliderCaptcha');
            var options = typeof option === 'object' && option;
 
            if (data && !/reset/.test(option)) return;
            if (!data) $this.data('lgb.SliderCaptcha', data = new SliderCaptcha(this, options));
            if (typeof option === 'string') data[option]();
        });
    }
 
    $.fn.sliderCaptcha = Plugin;
    $.fn.sliderCaptcha.Constructor = SliderCaptcha;
 
    var _proto = SliderCaptcha.prototype;
    _proto.init = function () {
        this.initDOM();
        this.initImg();
        this.bindEvents();
    };
 
    _proto.initDOM = function () {
        var createElement = function (tagName, className, val) {
            val = val || '';
            var elment = document.createElement(tagName);
            elment.className = className;
            elment.innerHTML  = val;
            return elment;
        };
 
        var createCanvas = function (width, height) {
            var canvas = document.createElement('canvas');
            canvas.width = width;
            canvas.height = height;
            return canvas;
        };
 
        var canvas = createCanvas(this.options.width - 2, this.options.height) // 画布
        var block = canvas.cloneNode(true) // 滑块
        var sliderContainer = createElement('div', 'sliderContainer');
        var refreshIcon = createElement('i', 'refreshIcon',
            '<svg t="1649226376231" class="icon" viewBox="0 0 1080 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="2432" width="16" height="16"><path d="M750.933333 113.777778C659.911111 62.577778 551.822222 39.822222 455.111111 56.888889c-22.755556 5.688889-39.822222 22.755556-28.444444 45.511111 5.688889 17.066667 22.755556 28.444444 39.822222 28.444444 85.333333-11.377778 170.666667 5.688889 250.311111 51.2 182.044444 102.4 250.311111 329.955556 164.977778 512-5.688889 17.066667-17.066667 28.444444-17.066667 28.444445l-79.644444-45.511111-17.066667 256c5.688889 5.688889 244.622222-119.466667 244.622222-119.466667l-79.644444-45.511111 17.066666-34.133333c96.711111-227.555556 11.377778-494.933333-199.111111-620.088889zM312.888889 870.4C119.466667 762.311111 56.888889 517.688889 164.977778 324.266667l85.333333 45.511111 17.066667-250.311111L22.755556 238.933333l79.644444 45.511111c-130.844444 233.244444-39.822222 523.377778 170.666667 648.533334 96.711111 56.888889 204.8 73.955556 307.2 56.888889 22.755556-5.688889 34.133333-22.755556 28.444444-45.511111-5.688889-17.066667-22.755556-28.444444-39.822222-28.444445-85.333333 17.066667-176.355556 0-256-45.511111z" fill="#999999" p-id="2433"></path></svg>');
        var sliderMask = createElement('div', 'sliderMask');
        var sliderbg = createElement('div', 'sliderbg');
        var slider = createElement('div', 'slider');
        var sliderIcon = createElement('i', 'sliderIcon',
            '<svg t="1649226619443" class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="3494" width="28" height="28"><path d="M909.312 506.24l-127.616-100.608a7.232 7.232 0 0 0-11.712 5.696v64.64H547.968V254.08h64.832c6.016 0 9.408-7.04 5.76-11.712L517.632 114.688a7.168 7.168 0 0 0-11.328 0L405.632 242.304a7.232 7.232 0 0 0 5.696 11.712h64.64v222.016H254.08V411.2a7.232 7.232 0 0 0-11.712-5.76L114.688 506.368a7.168 7.168 0 0 0 0 11.328l127.488 100.8a7.232 7.232 0 0 0 11.712-5.76v-64.64h222.016v221.952H411.072a7.232 7.232 0 0 0-5.696 11.712l100.8 127.488a7.168 7.168 0 0 0 11.328 0l100.8-127.488a7.232 7.232 0 0 0-5.76-11.712H548.096V547.968h221.952v64.832c0 6.016 7.04 9.408 11.712 5.76l127.488-100.864a7.296 7.296 0 0 0 0.128-11.392z" fill="#515151" fill-opacity=".65" p-id="3495"></path></svg>');
        var text = createElement('span', 'sliderText');
 
        block.className = 'block'
        text.innerHTML = this.options.barText;
 
        var el = this.$element;
        el.append($(canvas));
        el.append($(refreshIcon));
        el.append($(block));
        slider.appendChild(sliderIcon);
        sliderMask.appendChild(slider);
        sliderContainer.appendChild(sliderbg);
        sliderContainer.appendChild(sliderMask);
        sliderContainer.appendChild(text);
        el.append($(sliderContainer));
 
        Object.assign(this, {
            canvas,
            block,
            sliderContainer: $(sliderContainer),
            refreshIcon,
            slider,
            sliderMask,
            sliderIcon,
            text: $(text),
            canvasCtx: canvas.getContext('2d'),
            blockCtx: block.getContext('2d')
        })
    };
 
    _proto.initImg = function () {
        var that = this;
        var isIE = window.navigator.userAgent.indexOf('Trident') > -1;
        var L = this.options.sliderL + this.options.sliderR * 2 + 3; // 滑块实际边长
        var drawImg = function (ctx, operation) {
            var l = that.options.sliderL;
            var r = that.options.sliderR;
            var PI = that.options.PI;
            var x = that.x;
            var y = that.y;
            ctx.beginPath()
            ctx.moveTo(x, y)
            ctx.arc(x + l / 2, y - r + 2, r, 0.72 * PI, 2.26 * PI)
            ctx.lineTo(x + l, y)
            ctx.arc(x + l + r - 2, y + l / 2, r, 1.21 * PI, 2.78 * PI)
            ctx.lineTo(x + l, y + l)
            ctx.lineTo(x, y + l)
            ctx.arc(x + r - 2, y + l / 2, r + 0.4, 2.76 * PI, 1.24 * PI, true)
            ctx.lineTo(x, y)
            ctx.lineWidth = 2
            ctx.fillStyle = 'rgba(255, 255, 255, 0.7)'
            ctx.strokeStyle = 'rgba(255, 255, 255, 0.7)'
            ctx.stroke()
            ctx[operation]()
            ctx.globalCompositeOperation = isIE ? 'xor' : 'overlay'
        }
 
        var getRandomNumberByRange = function (start, end) {
            return Math.round(Math.random() * (end - start) + start);
        };
        var img = new window.Image();
        img.crossOrigin = 'Anonymous';
        var loadCount = 0;
        img.onload = function () {
            // 随机创建滑块的位置
            that.x = getRandomNumberByRange(L + 10, that.options.width - (L + 10));
            that.y = getRandomNumberByRange(10 + that.options.sliderR * 2, that.options.height - (L + 10));
            drawImg(that.canvasCtx, 'fill');
            drawImg(that.blockCtx, 'clip');
 
            that.canvasCtx.drawImage(img, 0, 0, that.options.width - 2, that.options.height);
            that.blockCtx.drawImage(img, 0, 0, that.options.width - 2, that.options.height);
            var y = that.y - that.options.sliderR * 2 - 1;
            var ImageData = that.blockCtx.getImageData(that.x - 3, y, L, L);
            that.block.width = L;
            that.blockCtx.putImageData(ImageData, 0, y);
            that.text.text(that.text.attr('data-text'));
        };
        img.onerror = function () {
            loadCount++;
            if (window.location.protocol === 'file:') {
                loadCount = that.options.maxLoadCount;
                console.error('can\'t load pic resource file from File protocal. Please try http or https');
            }
            if (loadCount >= that.options.maxLoadCount) {
                that.text.text('加载失败').addClass('text-danger');
                return;
            }
            img.src = that.options.localImages();
        };
        img.setSrc = function () {
            var src = '';
            loadCount = 0;
            that.text.removeClass('text-danger');
            src = '/assets/libs/yzm/images/'+ Math.round(Math.random() * 100) + '.jpg';
            if (isIE) { // IE浏览器无法通过img.crossOrigin跨域，使用ajax获取图片blob然后转为dataURL显示
                var xhr = new window.XMLHttpRequest()
                xhr.onloadend = function (e) {
                    var file = new window.FileReader(); // FileReader仅支持IE10+
                    file.readAsDataURL(e.target.response);
                    file.onloadend = function (e) {
                        img.src = e.target.result;
                    }
                }
                xhr.open('GET', src);
                xhr.responseType = 'blob';
                xhr.send();
            } else img.src = src;
        };
        img.setSrc();
        this.text.attr('data-text', this.options.barText);
        this.text.text(this.options.loadingText);
        this.img = img
    };
 
    _proto.clean = function () {
        this.canvasCtx.clearRect(0, 0, this.options.width, this.options.height);
        this.blockCtx.clearRect(0, 0, this.options.width, this.options.height);
        this.block.width = this.options.width;
    };
 
    _proto.bindEvents = function () {
        var that = this;
        this.$element.on('selectstart', function (e) {
            e.preventDefault();
            return false;
        });
 
        $(this.refreshIcon).on('click', function () {
            that.text.text(that.options.barText);
            that.reset();
            if ($.isFunction(that.options.onRefresh)) that.options.onRefresh.call(that.$element);
        });
 
        var originX, originY, trail = [],
            isMouseDown = false;
 
        var handleDragStart = function (e) {
            e.preventDefault();
            if (that.text.hasClass('text-danger')) return;
            originX = e.clientX || e.touches[0].clientX;
            originY = e.clientY || e.touches[0].clientY;
            isMouseDown = true;
            addDragEven();
        };
 
        var handleDragMove = function (e) {
            console.log('handleDragMove');
            if (!isMouseDown) return false;
            var eventX = e.clientX || e.touches[0].clientX;
            var eventY = e.clientY || e.touches[0].clientY;
            var moveX = eventX - originX;
            var moveY = eventY - originY;
            if (moveX < 0 || moveX + 40 > that.options.width) return false;
            that.slider.style.left = (moveX - 1) + 'px';
            var blockLeft = (that.options.width - 40 - 20) / (that.options.width - 40) * moveX;
            that.block.style.left = blockLeft + 'px';
 
            that.sliderContainer.addClass('sliderContainer_active');
            that.sliderMask.style.width = (moveX + 4) + 'px';
            trail.push(moveY);
        };

        var removeEven = function () {
            document.removeEventListener('mousemove', handleDragMove);
            document.removeEventListener('touchmove', handleDragMove);
            document.removeEventListener('mouseup', handleDragEnd);
            document.removeEventListener('touchend', handleDragEnd);
            document.removeEventListener('mousedown', returnFalse);
            document.removeEventListener('touchstart', returnFalse);
        };
        var handleDragEnd = function (e) {
            if (!isMouseDown) return false
            isMouseDown = false
            var eventX = e.clientX || e.changedTouches[0].clientX
            if (eventX == originX) return false
            that.sliderContainer.removeClass('sliderContainer_active');
            that.trail = trail
            var {
                spliced,
                verified
            } = that.verify()
            if (spliced && verified) {
                that.sliderContainer.addClass('sliderContainer_success');
                if ($.isFunction(that.options.onSuccess)) {
                    that.options.onSuccess.call(this, that);
                }
            } else {
                that.sliderContainer.addClass('sliderContainer_fail');
                if ($.isFunction(that.options.onFail)) that.options.onFail.call(this, that.$element);
                setTimeout(() => {
                    that.text.text(that.options.failedText);
                    that.reset();
                }, 500);
            }
            removeEven();
        };
 
        this.slider.addEventListener('mousedown', handleDragStart);
        this.slider.addEventListener('touchstart', handleDragStart);

        var returnFalse = function () { return false; };
        var addDragEven = function () {
            document.addEventListener('mousemove', handleDragMove);
            document.addEventListener('touchmove', handleDragMove);
            document.addEventListener('mouseup', handleDragEnd);
            document.addEventListener('touchend', handleDragEnd);
            document.addEventListener('mousedown', returnFalse);
            document.addEventListener('touchstart', returnFalse);
        };
    };
 
    _proto.verify = function () {
        var sum = function (x, y) { return x + y; };
        var square = function (x) { return x * x; };
        var arr = this.trail // 拖动时y轴的移动距离
        var average = arr.reduce(sum) / arr.length;
        var deviations = arr.map(function (x) { return x - average; });
        var stddev = Math.sqrt(deviations.map(square).reduce(sum) / arr.length);
        var left = parseInt(this.block.style.left);
        return {
            spliced: Math.abs(left - this.x) < this.options.offset,
            verified: stddev !== 0, // 简单验证下拖动轨迹，为零时表示Y轴上下没有波动，可能非人为操作
        }
    };
 
    _proto.reset = function () {
        this.sliderContainer.removeClass('sliderContainer_fail sliderContainer_success');
        this.slider.style.left = 0
        this.block.style.left = 0
        this.sliderMask.style.width = 0
        this.clean()
        this.text.attr('data-text', this.text.text());
        this.text.text(this.options.loadingText);
        this.img.setSrc();
    };
})(jQuery);
